@@ -38,10 +38,11 @@ from spyder.utils import icon_manager as ima
 from spyder.utils import syntaxhighlighters
 from spyder.utils.misc import getcwd_or_home
 from spyder.widgets.colors import ColorLayout
+from spyder.widgets.comboboxes import FileComboBox
 from spyder.widgets.sourcecode.codeeditor import CodeEditor
 
 
-HDPI_QT_PAGE = "http://doc.qt.io/qt-5/highdpi.html"
+HDPI_QT_PAGE = "https://doc.qt.io/qt-5/highdpi.html"
 
 
 class ConfigAccessMixin(object):
@@ -342,7 +343,11 @@ class SpyderConfigPage(ConfigPage, ConfigAccessMixin):
                 # integer for example (see qtpy.compat.from_qvariant):
                 if to_text_string(data) == to_text_string(value):
                     break
-            combobox.setCurrentIndex(index)
+            else:
+                if combobox.count() == 0:
+                    index = None
+            if index:
+                combobox.setCurrentIndex(index)
             combobox.currentIndexChanged.connect(lambda _foo, opt=option:
                                                  self.has_been_modified(opt))
             if combobox.restart_required:
@@ -676,7 +681,52 @@ class SpyderConfigPage(ConfigPage, ConfigAccessMixin):
         combobox.restart_required = restart
         combobox.label_text = text
         return widget
-    
+
+    def create_file_combobox(self, text, choices, option, default=NoDefault,
+                             tip=None, restart=False, filters=None,
+                             adjust_to_contents=False,
+                             default_line_edit=False):
+        """choices: couples (name, key)"""
+        combobox = FileComboBox(self, adjust_to_contents=adjust_to_contents,
+                                default_line_edit=default_line_edit)
+        combobox.restart_required = restart
+        combobox.label_text = text
+        edit = combobox.lineEdit()
+        edit.label_text = text
+        edit.restart_required = restart
+        self.lineedits[edit] = (option, default)
+
+        if tip is not None:
+            combobox.setToolTip(tip)
+        for name, key in choices:
+            if not (name is None and key is None):
+                combobox.addItem(name, to_qvariant(key))
+        # Insert separators
+        count = 0
+        for index, item in enumerate(choices):
+            name, key = item
+            if name is None and key is None:
+                combobox.insertSeparator(index + count)
+                count += 1
+        self.comboboxes[combobox] = (option, default)
+
+        msg = _('Invalid file path')
+        self.validate_data[edit] = (osp.isfile, msg)
+        browse_btn = QPushButton(ima.icon('FileIcon'), '', self)
+        browse_btn.setToolTip(_("Select file"))
+        browse_btn.clicked.connect(lambda: self.select_file(edit, filters))
+
+        layout = QGridLayout()
+        layout.addWidget(combobox, 0, 0, 0, 9)
+        layout.addWidget(browse_btn, 0, 10)
+        layout.setContentsMargins(0, 0, 0, 0)
+        widget = QWidget(self)
+        widget.combobox = combobox
+        widget.browse_btn = browse_btn
+        widget.setLayout(layout)
+
+        return widget
+
     def create_fontgroup(self, option=None, text=None, title=None,
                          tip=None, fontfilters=None, without_group=False):
         """Option=None -> setting plugin font"""
