@@ -20,6 +20,7 @@ import os
 import os.path as osp
 import uuid
 import sys
+import traceback
 
 # Third party imports
 from jupyter_client.connect import find_connection_file
@@ -42,8 +43,7 @@ except ImportError:
 
 # Local imports
 from spyder import dependencies
-from spyder.config.base import (_, DEV, get_conf_path, get_home_dir,
-                                get_module_path, running_under_pytest)
+from spyder.config.base import _, get_conf_path, get_home_dir
 from spyder.config.main import CONF
 from spyder.plugins import SpyderPluginWidget
 from spyder.plugins.configdialog import PluginConfigPage
@@ -77,6 +77,10 @@ IPYTHON_REQVER = ">=4.0;<6.0" if PY2 else ">=4.0"
 dependencies.add("IPython", _("IPython interactive python environment"),
                  required_version=IPYTHON_REQVER)
 
+MATPLOTLIB_REQVER = '>=2.0.0'
+dependencies.add("matplotlib", _("Display 2D graphics in the IPython Console"),
+                 required_version=MATPLOTLIB_REQVER, optional=True)
+
 #------------------------------------------------------------------------------
 # Existing kernels
 #------------------------------------------------------------------------------
@@ -396,7 +400,7 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         backend_group.setLayout(backend_layout)
         backend_group.setEnabled(self.get_option('pylab'))
         pylab_box.toggled.connect(backend_group.setEnabled)
-        
+
         # Inline backend Group
         inline_group = QGroupBox(_("Inline backend"))
         inline_label = QLabel(_("Decide how to render the figures created by "
@@ -404,7 +408,8 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         inline_label.setWordWrap(True)
         formats = (("PNG", 0), ("SVG", 1))
         format_box = self.create_combobox(_("Format:")+"   ", formats,
-                                       'pylab/inline/figure_format', default=0)
+                                          'pylab/inline/figure_format',
+                                          default=0)
         resolution_spin = self.create_spinbox(
                         _("Resolution:")+"  ", " "+_("dpi"),
                         'pylab/inline/resolution', min_=50, max_=999, step=0.1,
@@ -418,7 +423,15 @@ class IPythonConsoleConfigPage(PluginConfigPage):
                           _("Height:")+"  ", " "+_("inches"),
                           'pylab/inline/height', min_=1, max_=20, step=1,
                           tip=_("Default is 4"))
-        
+        bbox_inches_box = newcb(
+            _("Use a tight layout for inline plots"),
+            'pylab/inline/bbox_inches',
+            tip=_("Sets bbox_inches to \"tight\" when\n"
+                  "plotting inline with matplotlib.\n"
+                  "When enabled, can cause discrepancies\n"
+                  "between the image displayed inline and\n"
+                  "that created using savefig."))
+
         inline_v_layout = QVBoxLayout()
         inline_v_layout.addWidget(inline_label)
         inline_layout = QGridLayout()
@@ -433,6 +446,8 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         inline_layout.addWidget(height_spin.plabel, 4, 0)
         inline_layout.addWidget(height_spin.spinbox, 4, 1)
         inline_layout.addWidget(height_spin.slabel, 4, 2)
+        inline_layout.addWidget(bbox_inches_box, 5, 0, 1, 4)
+
         inline_h_layout = QHBoxLayout()
         inline_h_layout.addLayout(inline_layout)
         inline_h_layout.addStretch(1)
@@ -477,20 +492,46 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         run_file_group.setLayout(run_file_layout)
         
         # ---- Advanced settings ----
+        # Enable Jedi completion
+        jedi_group = QGroupBox(_("Jedi completion"))
+        jedi_label = QLabel(_("Enable Jedi-based <tt>Tab</tt> completion "
+                                  "in the IPython console; similar to the "
+                                  "greedy completer, but without evaluating "
+                                  "the code.<br>"
+                                  "<b>Warning:</b> Slows down your console "
+                                  "when working with large dataframes!"))
+        jedi_label.setWordWrap(True)
+        jedi_box = newcb(_("Use Jedi completion in the IPython console"), 
+                             "jedi_completer",
+                             tip="<b>Warning</b>: "
+                                 "Slows down your console when working with "
+                                 "large dataframes!<br>"
+                                 "Allows completion of nested lists etc.")
+        
+        jedi_layout = QVBoxLayout()
+        jedi_layout.addWidget(jedi_label)
+        jedi_layout.addWidget(jedi_box)
+        jedi_group.setLayout(jedi_layout)
+                
         # Greedy completer group
         greedy_group = QGroupBox(_("Greedy completion"))
         greedy_label = QLabel(_("Enable <tt>Tab</tt> completion on elements "
                                 "of lists, results of function calls, etc, "
-                                "<i>without</i> assigning them to a "
-                                "variable.<br>"
-                                "For example, you can get completions on "
-                                "things like <tt>li[0].&lt;Tab&gt;</tt> or "
-                                "<tt>ins.meth().&lt;Tab&gt;</tt>"))
+                                "<i>without</i> assigning them to a variable, "
+                                "like <tt>li[0].&lt;Tab&gt;</tt> or "
+                                "<tt>ins.meth().&lt;Tab&gt;</tt> <br>"
+                                "<b>Warning:</b> Due to a bug, IPython's "
+                                "greedy completer requires a leading "
+                                "<tt>&lt;Space&gt;</tt> for some completions; "
+                                "e.g.  <tt>np.sin(&lt;Space&gt;np.&lt;Tab&gt;"
+                                "</tt> works while <tt>np.sin(np.&lt;Tab&gt; "
+                                "</tt> doesn't."))
         greedy_label.setWordWrap(True)
-        greedy_box = newcb(_("Use the greedy completer"), "greedy_completer",
+        greedy_box = newcb(_("Use greedy completion in the IPython console"),
+                           "greedy_completer",
                            tip="<b>Warning</b>: It can be unsafe because the "
-                                "code is actually evaluated when you press "
-                                "<tt>Tab</tt>.")
+                               "code is actually evaluated when you press "
+                               "<tt>Tab</tt>.")
         
         greedy_layout = QVBoxLayout()
         greedy_layout.addWidget(greedy_label)
@@ -500,10 +541,10 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         # Autocall group
         autocall_group = QGroupBox(_("Autocall"))
         autocall_label = QLabel(_("Autocall makes IPython automatically call "
-                                "any callable object even if you didn't type "
-                                "explicit parentheses.<br>"
-                                "For example, if you type <i>str 43</i> it "
-                                "becomes <i>str(43)</i> automatically."))
+                                  "any callable object even if you didn't "
+                                  "type explicit parentheses.<br>"
+                                  "For example, if you type <i>str 43</i> it "
+                                  "becomes <i>str(43)</i> automatically."))
         autocall_label.setWordWrap(True)
         
         smart = _('Smart')
@@ -575,7 +616,7 @@ class IPythonConsoleConfigPage(PluginConfigPage):
                                     _("Graphics"))
         tabs.addTab(self.create_tab(run_lines_group, run_file_group),
                                     _("Startup"))
-        tabs.addTab(self.create_tab(greedy_group, autocall_group, sympy_group,
+        tabs.addTab(self.create_tab(jedi_group, greedy_group, autocall_group, sympy_group,
                                     prompts_group), _("Advanced Settings"))
 
         vlayout = QVBoxLayout()
@@ -914,7 +955,18 @@ class IPythonConsole(SpyderPluginWidget):
                     line += " %s" % norm(args)
 
             try:
-                if current_client:
+                if client.shellwidget._executing:
+                    # Don't allow multiple executions when there's
+                    # still an execution taking place
+                    # Fixes issue 7293
+                    pass
+                elif client.shellwidget._reading:
+                    client.shellwidget._append_html(
+                        _("<br><b>Please exit from debugging before trying to "
+                          "run a file in this console.</b>\n<hr><br>"),
+                        before_prompt=True)
+                    return
+                elif current_client:
                     self.execute_code(line, current_client, clear_variables)
                 else:
                     if is_new_client:
@@ -1023,24 +1075,22 @@ class IPythonConsole(SpyderPluginWidget):
         # Else we won't be able to create a client
         if not CONF.get('main_interpreter', 'default'):
             pyexec = CONF.get('main_interpreter', 'executable')
-            has_ipykernel = programs.is_module_installed('ipykernel',
-                                                         interpreter=pyexec)
-            has_cloudpickle = programs.is_module_installed('cloudpickle',
-                                                           interpreter=pyexec)
-            if not (has_ipykernel and has_cloudpickle):
-                client.show_kernel_error(_("Your Python environment or "
-                                     "installation doesn't "
-                                     "have the <tt>ipykernel</tt> and "
-                                     "<tt>cloudpickle</tt> modules "
-                                     "installed on it. Without these modules "
-                                     "is not possible for Spyder to create a "
-                                     "console for you.<br><br>"
-                                     "You can install them by running "
-                                     "in a system terminal:<br><br>"
-                                     "<tt>pip install ipykernel cloudpickle</tt>"
-                                     "<br><br>"
-                                     "or<br><br>"
-                                     "<tt>conda install ipykernel cloudpickle</tt>"))
+            has_spyder_kernels = programs.is_module_installed(
+                                                        'spyder_kernels',
+                                                        interpreter=pyexec,
+                                                        version='<1.0.0')
+            if not has_spyder_kernels:
+                client.show_kernel_error(
+                        _("Your Python environment or installation doesn't "
+                          "have the <tt>spyder-kernels</tt> module or the "
+                          "right version of it installed. "
+                          "Without this module is not possible for "
+                          "Spyder to create a console for you.<br><br>"
+                          "You can install it by running in a system terminal:"
+                          "<br><br>"
+                          "<tt>conda install spyder-kernels=0.*</tt>"
+                          "<br><br>or<br><br>"
+                          "<tt>pip install spyder-kernels==0.*</tt>"))
                 return
 
         self.connect_client_to_kernel(client, is_cython=is_cython)
@@ -1087,39 +1137,13 @@ class IPythonConsole(SpyderPluginWidget):
         shellwidget.kernel_manager = km
         shellwidget.kernel_client = kc
 
-    def set_editor(self):
-        """Set the editor used by the %edit magic"""
-        # Get Python executable used by Spyder
-        python = sys.executable
-        if PY2:
-            python = encoding.to_unicode_from_fs(python)
-
-        # Compose command for %edit
-        spy_dir = osp.dirname(get_module_path('spyder'))
-        if DEV:
-            bootstrap = osp.join(spy_dir, 'bootstrap.py')
-            if PY2:
-                bootstrap = encoding.to_unicode_from_fs(bootstrap)
-            editor = u'"{0}" "{1}" --'.format(python, bootstrap)
-        else:
-            import1 = "import sys"
-            # We need to add spy_dir to sys.path so this test can be
-            # run in our CIs
-            if running_under_pytest():
-                if os.name == 'nt':
-                    import1 = (import1 +
-                               '; sys.path.append(""{}"")'.format(spy_dir))
-                else:
-                    import1 = (import1 +
-                               "; sys.path.append('{}')".format(spy_dir))
-            import2 = "from spyder.app.start import send_args_to_spyder"
-            code = "send_args_to_spyder([sys.argv[-1]])"
-            editor = u"\"{0}\" -c \"{1}; {2}; {3}\"".format(python,
-                                                            import1,
-                                                            import2,
-                                                            code)
-
-        return editor
+    @Slot(object, object)
+    def edit_file(self, filename, line):
+        """Handle %edit magic petitions."""
+        if encoding.is_text_file(filename):
+            # The default line number sent by ipykernel is always the last
+            # one, but we prefer to use the first.
+            self.edit_goto.emit(filename, 1, '')
 
     def config_options(self):
         """
@@ -1179,10 +1203,6 @@ class IPythonConsole(SpyderPluginWidget):
         spy_cfg.JupyterWidget.style_sheet = style_sheet
         spy_cfg.JupyterWidget.syntax_style = color_scheme
 
-        # Editor for %edit
-        if CONF.get('main', 'single_instance'):
-            spy_cfg.JupyterWidget.editor = self.set_editor()
-
         # Merge QtConsole and Spyder configs. Spyder prefs will have
         # prevalence over QtConsole ones
         cfg._merge(spy_cfg)
@@ -1251,6 +1271,9 @@ class IPythonConsole(SpyderPluginWidget):
                               lambda fname, lineno, shellwidget=shellwidget:
                               self.pdb_has_stopped(fname, lineno, shellwidget))
 
+        # To handle %edit magic petitions
+        shellwidget.custom_edit_requested.connect(self.edit_file)
+
         # Set shell cwd according to preferences
         cwd_path = ''
         if CONF.get('workingdir', 'console/use_project_or_home_directory'):
@@ -1495,18 +1518,15 @@ class IPythonConsole(SpyderPluginWidget):
         """Create kernel manager and client."""
         # Kernel spec
         kernel_spec = self.create_kernel_spec(is_cython=is_cython)
-        if not kernel_spec.env.get('PYTHONPATH'):
-            error_msg = _("This error was most probably caused by installing "
-                          "Spyder in a directory with non-ascii characters "
-                          "(i.e. characters with tildes, apostrophes or "
-                          "non-latin symbols).<br><br>"
-                          "To fix it, please <b>reinstall</b> Spyder in a "
-                          "different location.")
-            return (error_msg, None)
 
         # Kernel manager
-        kernel_manager = QtKernelManager(connection_file=connection_file,
-                                         config=None, autorestart=True)
+        try:
+            kernel_manager = QtKernelManager(connection_file=connection_file,
+                                             config=None, autorestart=True)
+        except Exception:
+            error_msg = _("The error is:<br><br>"
+                          "<tt>{}</tt>").format(traceback.format_exc())
+            return (error_msg, None)
         kernel_manager._kernel_spec = kernel_spec
 
         # Save stderr in a file to read it later in case of errors
@@ -1779,9 +1799,9 @@ class IPythonConsole(SpyderPluginWidget):
                 return
 
         # Assign kernel manager and client to shellwidget
+        kernel_client.start_channels()
         client.shellwidget.kernel_client = kernel_client
         client.shellwidget.kernel_manager = kernel_manager
-        kernel_client.start_channels()
         if external_kernel:
             client.shellwidget.sig_is_spykernel.connect(
                     self.connect_external_kernel)
