@@ -1,9 +1,16 @@
 # -*- coding: utf-8 -*-
-#
+# -----------------------------------------------------------------------------
 # Copyright © Spyder Project Contributors
-# Licensed under the terms of the MIT License
 #
+# Licensed under the terms of the MIT License
+# (see spyder/__init__.py for details)
+# -----------------------------------------------------------------------------
 
+"""
+Tests for the IPython console plugin.
+"""
+
+# Standard library imports
 import codecs
 import os
 import os.path as osp
@@ -11,99 +18,231 @@ import shutil
 import tempfile
 from textwrap import dedent
 
+# Third party imports
+import cloudpickle
 from flaky import flaky
-from ipykernel.serialize import serialize_object
 from pygments.token import Name
 import pytest
-from qtpy import PYQT4, PYQT5, PYQT_VERSION
-from qtpy.QtCore import Qt, QTimer
-from qtpy.QtWidgets import QApplication
+from qtpy import PYQT5
+from qtpy.QtCore import Qt
+import zmq
 
+# Local imports
 from spyder.config.gui import get_color_scheme
 from spyder.config.main import CONF
-from spyder.py3compat import PY2, PY3
-from spyder.plugins.ipythonconsole import (IPythonConsole,
-                                           KernelConnectionDialog)
-from spyder.utils.environ import listdict2envdict
+from spyder.py3compat import PY2, to_text_string
+from spyder.plugins.ipythonconsole import IPythonConsole
 from spyder.utils.ipython.style import create_style_class
-from spyder.utils.test import close_message_box
-from spyder.widgets.variableexplorer.collectionseditor import CollectionsEditor
+from spyder.utils.programs import TEMPDIR
 
 
-#==============================================================================
+# =============================================================================
 # Constants
-#==============================================================================
+# =============================================================================
 SHELL_TIMEOUT = 20000
-PYQT_WHEEL = PYQT_VERSION > '5.6'
 TEMP_DIRECTORY = tempfile.gettempdir()
+NON_ASCII_DIR = osp.join(TEMP_DIRECTORY, u'測試', u'اختبار')
 
 
-#==============================================================================
+# =============================================================================
 # Utillity Functions
-#==============================================================================
-def open_client_from_connection_info(connection_info, qtbot):
-    top_level_widgets = QApplication.topLevelWidgets()
-    for w in top_level_widgets:
-        if isinstance(w, KernelConnectionDialog):
-            w.cf.setText(connection_info)
-            qtbot.keyClick(w, Qt.Key_Enter)
-
+# =============================================================================
 def get_console_font_color(syntax_style):
     styles = create_style_class(syntax_style).styles
     font_color = styles[Name]
-
     return font_color
 
+
 def get_console_background_color(style_sheet):
     background_color = style_sheet.split('background-color:')[1]
     background_color = background_color.split(';')[0]
     return background_color
 
-#==============================================================================
+
+# =============================================================================
 # Qt Test Fixtures
-#==============================================================================
+# =============================================================================
 @pytest.fixture
-def ipyconsole(request):
-    try:
-        console = IPythonConsole(None, testing=True, test_dir=request.param)
-    except AttributeError:
-        console = IPythonConsole(None, testing=True)
+def ipyconsole(qtbot, request):
+    """IPython console fixture."""
+    # Tests assume inline backend
+    CONF.set('ipython_console', 'pylab/backend', 0)
+
+    # Test the console with a non-ascii temp dir
+    non_ascii_dir = request.node.get_marker('non_ascii_dir')
+    if non_ascii_dir:
+        test_dir = NON_ASCII_DIR
+    else:
+        test_dir = TEMPDIR
+
+    # Instruct the console to not use a stderr file
+    no_stderr_file = request.node.get_marker('no_stderr_file')
+    if no_stderr_file:
+        test_no_stderr = True
+    else:
+        test_no_stderr = False
+
+    # Use the automatic backend if requested
+    auto_backend = request.node.get_marker('auto_backend')
+    if auto_backend:
+        CONF.set('ipython_console', 'pylab/backend', 1)
+
+    # Create the console and a new client
+    console = IPythonConsole(parent=None,
+                             testing=True,
+                             test_dir=test_dir,
+                             test_no_stderr=test_no_stderr)
     console.create_new_client()
+
+    # Close callback
     def close_console():
         console.closing_plugin()
         console.close()
     request.addfinalizer(close_console)
+
+    qtbot.addWidget(console)
     console.show()
     return console
 
 
-#==============================================================================
+# =============================================================================
 # Tests
-#==============================================================================
-@pytest.mark.parametrize('ipyconsole', [osp.join(TEMP_DIRECTORY, u'測試',
-                                                 u'اختبار')], indirect=True)
+# =============================================================================
+@pytest.mark.slow
+@flaky(max_runs=3)
+@pytest.mark.auto_backend
 @pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_console_stderr_file(ipyconsole, qtbot):
-    """Test a the creation of a console with a stderr file in ascii dir."""
+def test_auto_backend(ipyconsole, qtbot):
+    """Test that the automatic backend is working correctly."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
 
-    # Create a new client with s stderr file in a non-ascii dir
-    ipyconsole.create_new_client()
+    # This is here to generate further errors
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("%matplotlib qt5")
+
+    # Assert there are no errors in the console
+    control = ipyconsole.get_focus_widget()
+    assert 'NOTE' not in control.toPlainText()
+    assert 'Error' not in control.toPlainText()
+
+
+@pytest.mark.slow
+@flaky(max_runs=3)
+def test_tab_rename_for_slaves(ipyconsole, qtbot):
+    """Test slave clients are renamed correctly."""
+    # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
+
+    cf = ipyconsole.get_current_client().connection_file
+    ipyconsole._create_client_for_kernel(cf, None, None, None)
+    qtbot.wait(1000)
+
+    # Rename slave
+    ipyconsole.rename_tabs_after_change('foo')
+
+    # Assert both clients have the same name
+    assert 'foo' in ipyconsole.get_clients()[0].get_name()
+    assert 'foo' in ipyconsole.get_clients()[1].get_name()
+
+
+@pytest.mark.slow
+@flaky(max_runs=3)
+def test_no_repeated_tabs_name(ipyconsole, qtbot):
+    """Test that tabs can't have repeated given names."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
+
+    # Rename first client
+    ipyconsole.rename_tabs_after_change('foo')
+
+    # Create a new client and try to rename it
+    ipyconsole.create_new_client()
+    ipyconsole.rename_tabs_after_change('foo')
+
+    # Assert the rename didn't take place
+    client_name = ipyconsole.get_current_client().get_name()
+    assert '2' in client_name
+
+
+@pytest.mark.slow
+@flaky(max_runs=3)
+def test_tabs_preserve_name_after_move(ipyconsole, qtbot):
+    """Test that tabs preserve their names after they are moved."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
+
+    # Create a new client
+    ipyconsole.create_new_client()
+
+    # Move tabs
+    ipyconsole.tabwidget.tabBar().moveTab(0, 1)
+
+    # Assert the second client is in the first position
+    client_name = ipyconsole.get_clients()[0].get_name()
+    assert '2' in client_name
+
+
+@pytest.mark.slow
+@flaky(max_runs=3)
+def test_conf_env_vars(ipyconsole, qtbot):
+    """Test that kernels have env vars set by our kernel spec."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Get a CONF env var
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("import os; a = os.environ.get('SPY_SYMPY_O')")
+
+    # Assert we get the assigned value correctly
+    assert shell.get_value('a') == 'False'
+
+
+@pytest.mark.slow
+@flaky(max_runs=3)
+@pytest.mark.no_stderr_file
+def test_no_stderr_file(ipyconsole, qtbot):
+    """Test that consoles can run without an stderr."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Execute a simple assignment
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = 1')
 
-    # Assert we get the a value correctly
+    # Assert we get the assigned value correctly
     assert shell.get_value('a') == 1
 
 
+@pytest.mark.slow
+@pytest.mark.non_ascii_dir
+@flaky(max_runs=3)
+@pytest.mark.skipif(os.name == 'nt', reason="It fails on Windows")
+def test_non_ascii_stderr_file(ipyconsole, qtbot):
+    """Test the creation of a console with a stderr file in a non-ascii dir."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Execute a simple assignment
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('a = 1')
+
+    # Assert we get the assigned value
+    assert shell.get_value('a') == 1
+
+
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
 def test_console_import_namespace(ipyconsole, qtbot):
     """Test an import of the form 'from foo import *'."""
     # Wait until the window is fully up
@@ -118,9 +257,13 @@ def test_console_import_namespace(ipyconsole, qtbot):
     assert shell.get_value('e') == 2.718281828459045
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
 def test_console_disambiguation(ipyconsole, qtbot):
     """Test the disambiguation of dedicated consoles."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
+
     # Create directories and file for TEMP_DIRECTORY/a/b/c.py
     # and TEMP_DIRECTORY/a/d/c.py
     dir_b = osp.join(TEMP_DIRECTORY, 'a', 'b')
@@ -152,8 +295,12 @@ def test_console_disambiguation(ipyconsole, qtbot):
     assert client.get_name() == 'c.py - b/A'
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
 def test_console_coloring(ipyconsole, qtbot):
+    """Test that console gets the same coloring present in the Editor."""
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     config_options = ipyconsole.config_options()
 
@@ -176,10 +323,69 @@ def test_console_coloring(ipyconsole, qtbot):
     assert console_font_color.strip() == editor_font_color.strip()
 
 
+@pytest.mark.slow
+@flaky(max_runs=3)
+def test_set_cwd(ipyconsole, qtbot, tmpdir):
+    """Test kernel when changing cwd."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Issue 6451.
+    savetemp = shell._cwd
+    tempdir = to_text_string(tmpdir.mkdir("queen's"))
+    shell.set_cwd(tempdir)
+
+    # Get current directory.
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("import os; cwd = os.getcwd()")
+
+    # Assert we get the assigned value correctly
+    assert shell.get_value('cwd') == tempdir
+
+    # Restore original.
+    shell.set_cwd(savetemp)
+
+
+@pytest.mark.slow
+@flaky(max_runs=3)
+def test_get_cwd(ipyconsole, qtbot, tmpdir):
+    """Test current working directory."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Issue 6451.
+    savetemp = shell._cwd
+    tempdir = to_text_string(tmpdir.mkdir("queen's"))
+    assert shell._cwd != tempdir
+
+    # Need to escape \ on Windows.
+    if os.name == 'nt':
+        tempdir = tempdir.replace(u"\\", u"\\\\")
+
+    # Change directory in the console.
+    with qtbot.waitSignal(shell.executed):
+        shell.execute(u"import os; os.chdir(u'''{}''')".format(tempdir))
+
+    # Ask for directory.
+    with qtbot.waitSignal(shell.sig_change_cwd):
+        shell.get_cwd()
+
+    if os.name == 'nt':
+        tempdir = tempdir.replace(u"\\\\", u"\\")
+
+    assert shell._cwd == tempdir
+
+    shell.set_cwd(savetemp)
+
+
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_get_env(ipyconsole, qtbot):
-    """Test that showing env var contents is working as expected."""
+    """Test that getting env vars from the kernel is working as expected."""
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
@@ -188,54 +394,45 @@ def test_get_env(ipyconsole, qtbot):
         shell.execute("import os; os.environ['FOO'] = 'bar'" )
 
     # Ask for os.environ contents
-    with qtbot.waitSignal(shell.sig_show_env):
+    with qtbot.waitSignal(shell.sig_show_env) as blocker:
         shell.get_env()
 
-    # Get env contents from the generated widget
-    top_level_widgets = QApplication.topLevelWidgets()
-    for w in top_level_widgets:
-        if isinstance(w, CollectionsEditor):
-            env_contents = w.get_value()
-            qtbot.keyClick(w, Qt.Key_Enter)
+    # Get env contents from the signal
+    env_contents = blocker.args[0]
 
     # Assert that our added entry is part of os.environ
-    env_contents = listdict2envdict(env_contents)
     assert env_contents['FOO'] == 'bar'
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
-def test_get_syspath(ipyconsole, qtbot):
-    """Test that showing sys.path contents is working as expected."""
+@pytest.mark.skipif(os.name == 'nt',
+                    reason="Fails due to differences in path handling")
+def test_get_syspath(ipyconsole, qtbot, tmpdir):
+    """
+    Test that getting sys.path contents from the kernel is working as
+    expected.
+    """
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
     # Add a new entry to sys.path
     with qtbot.waitSignal(shell.executed):
-        tmp_dir = tempfile.mkdtemp()
-        shell.execute("import sys, tempfile; sys.path.append('%s')" % tmp_dir)
+        tmp_dir = to_text_string(tmpdir)
+        shell.execute("import sys; sys.path.append('%s')" % tmp_dir)
 
     # Ask for sys.path contents
-    with qtbot.waitSignal(shell.sig_show_syspath):
+    with qtbot.waitSignal(shell.sig_show_syspath) as blocker:
         shell.get_syspath()
 
-    # Get sys.path contents from the generated widget
-    top_level_widgets = QApplication.topLevelWidgets()
-    for w in top_level_widgets:
-        if isinstance(w, CollectionsEditor):
-            syspath_contents = w.get_value()
-            qtbot.keyClick(w, Qt.Key_Enter)
+    # Get sys.path contents from the signal
+    syspath_contents = blocker.args[0]
 
     # Assert that our added entry is part of sys.path
     assert tmp_dir in syspath_contents
 
-    # Remove temporary directory
-    try:
-        os.rmdir(tmp_dir)
-    except:
-        pass
-
 
+@pytest.mark.slow
 @flaky(max_runs=10)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_browse_history_dbg(ipyconsole, qtbot):
@@ -269,9 +466,9 @@ def test_browse_history_dbg(ipyconsole, qtbot):
     assert '!aa = 10' in control.toPlainText()
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or PY2,
-                    reason="It times out sometimes on Windows and doesn't work on PY2")
+@pytest.mark.skipif(PY2, reason="It doesn't work on PY2")
 def test_unicode_vars(ipyconsole, qtbot):
     """
     Test that the Variable Explorer Works with unicode variables.
@@ -288,11 +485,12 @@ def test_unicode_vars(ipyconsole, qtbot):
     assert shell.get_value('д') == 10
 
     # Change its value and verify
-    shell.set_value('д', serialize_object(20))
+    shell.set_value('д', [cloudpickle.dumps(20, protocol=2)])
     qtbot.wait(1000)
     assert shell.get_value('д') == 20
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
 def test_read_stderr(ipyconsole, qtbot):
     """
@@ -310,9 +508,12 @@ def test_read_stderr(ipyconsole, qtbot):
     assert content == client._read_stderr()
 
 
+@pytest.mark.slow
 @flaky(max_runs=10)
-@pytest.mark.skipif(os.environ.get('CI', None) is not None,
-                    reason="It times out in our CIs")
+@pytest.mark.no_xvfb
+@pytest.mark.skipif(os.environ.get('CI', None) is not None and os.name == 'nt',
+                    reason="It times out on AppVeyor.")
+@pytest.mark.timeout(timeout=20, method='thread')
 def test_values_dbg(ipyconsole, qtbot):
     """
     Test that getting, setting, copying and removing values is working while
@@ -338,7 +539,7 @@ def test_values_dbg(ipyconsole, qtbot):
     assert shell.get_value('aa') == 10
 
     # Set value
-    shell.set_value('aa', serialize_object(20))
+    shell.set_value('aa', [cloudpickle.dumps(20, protocol=2)])
     qtbot.wait(1000)
     assert shell.get_value('aa') == 20
 
@@ -356,6 +557,7 @@ def test_values_dbg(ipyconsole, qtbot):
     assert "*** NameError: name 'aa' is not defined" in control.toPlainText()
 
 
+@pytest.mark.slow
 @flaky(max_runs=10)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_plot_magic_dbg(ipyconsole, qtbot):
@@ -386,8 +588,8 @@ def test_plot_magic_dbg(ipyconsole, qtbot):
     assert shell._control.toHtml().count('img src') == 1
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_run_doctest(ipyconsole, qtbot):
     """
     Test that doctests can be run without problems
@@ -422,6 +624,7 @@ def test_run_doctest(ipyconsole, qtbot):
     assert "TestResults(failed=0, attempted=2)" in shell._control.toPlainText()
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt' or (PY2 and PYQT5),
                     reason="It times out frequently")
@@ -480,6 +683,7 @@ def test_ctrl_c_dbg(ipyconsole, qtbot):
     assert 'For copying text while debugging, use Ctrl+Shift+C' in control.toPlainText()
 
 
+@pytest.mark.slow
 @flaky(max_runs=10)
 @pytest.mark.skipif(os.name == 'nt', reason="It doesn't work on Windows")
 def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
@@ -502,7 +706,7 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
 
     # Test clear magic
     shell.clear_console()
-    qtbot.wait(500)
+    qtbot.wait(1000)
     assert '\nipdb> ' == control.toPlainText()
 
     # Test reset magic
@@ -521,8 +725,8 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     assert "*** NameError: name 'bb' is not defined" in control.toPlainText()
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_restart_kernel(ipyconsole, qtbot):
     """
     Test that kernel is restarted correctly
@@ -531,21 +735,21 @@ def test_restart_kernel(ipyconsole, qtbot):
     client = ipyconsole.get_current_client()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
-    # Do an assigment to verify that it's not there after restarting
+    # Do an assignment to verify that it's not there after restarting
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = 10')
 
     # Restart kernel and wait until it's up again
     shell._prompt_html = None
-    QTimer.singleShot(1000, lambda: close_message_box(qtbot))
     client.restart_kernel()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
+    assert 'Restarting kernel...' in shell._control.toPlainText()
     assert not shell.is_defined('a')
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_load_kernel_file_from_id(ipyconsole, qtbot):
     """
     Test that a new client is created using its id
@@ -557,18 +761,16 @@ def test_load_kernel_file_from_id(ipyconsole, qtbot):
     connection_file = osp.basename(client.connection_file)
     id_ = connection_file.split('kernel-')[-1].split('.json')[0]
 
-    QTimer.singleShot(2000, lambda: open_client_from_connection_info(
-                                        id_, qtbot))
-    ipyconsole.create_client_for_kernel()
+    ipyconsole._create_client_for_kernel(id_, None, None, None)
     qtbot.wait(1000)
 
     new_client = ipyconsole.get_clients()[1]
     assert new_client.id_ == dict(int_id='1', str_id='B')
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
-def test_load_kernel_file_from_location(ipyconsole, qtbot):
+def test_load_kernel_file_from_location(ipyconsole, qtbot, tmpdir):
     """
     Test that a new client is created using a connection file
     placed in a different location from jupyter_runtime_dir
@@ -577,23 +779,19 @@ def test_load_kernel_file_from_location(ipyconsole, qtbot):
     client = ipyconsole.get_current_client()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
-    connection_file = osp.join(tempfile.gettempdir(),
-                               osp.basename(client.connection_file))
+    fname = osp.basename(client.connection_file)
+    connection_file = to_text_string(tmpdir.join(fname))
     shutil.copy2(client.connection_file, connection_file)
 
-    QTimer.singleShot(2000, lambda: open_client_from_connection_info(
-                                        connection_file,
-                                        qtbot))
-    ipyconsole.create_client_for_kernel()
+    ipyconsole._create_client_for_kernel(connection_file, None, None, None)
     qtbot.wait(1000)
 
     assert len(ipyconsole.get_clients()) == 2
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or (PY3 and PYQT4),
-                    reason="It segfaults frequently")
-def test_load_kernel_file(ipyconsole, qtbot):
+def test_load_kernel_file(ipyconsole, qtbot, tmpdir):
     """
     Test that a new client is created using the connection file
     of an existing client
@@ -602,10 +800,7 @@ def test_load_kernel_file(ipyconsole, qtbot):
     client = ipyconsole.get_current_client()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
-    QTimer.singleShot(2000, lambda: open_client_from_connection_info(
-                                        client.connection_file,
-                                        qtbot))
-    ipyconsole.create_client_for_kernel()
+    ipyconsole._create_client_for_kernel(client.connection_file, None, None, None)
     qtbot.wait(1000)
 
     new_client = ipyconsole.get_clients()[1]
@@ -617,8 +812,8 @@ def test_load_kernel_file(ipyconsole, qtbot):
     assert shell.get_value('a') == new_shell.get_value('a')
 
 
+@pytest.mark.slow
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
 def test_sys_argv_clear(ipyconsole, qtbot):
     """Test that sys.argv is cleared up correctly"""
     shell = ipyconsole.get_current_shellwidget()
@@ -631,5 +826,36 @@ def test_sys_argv_clear(ipyconsole, qtbot):
     assert argv == ['']
 
 
+@pytest.mark.slow
+@flaky(max_runs=5)
+def test_set_elapsed_time(ipyconsole, qtbot):
+    """Test that the IPython console elapsed timer is set correctly."""
+    shell = ipyconsole.get_current_shellwidget()
+    client = ipyconsole.get_current_client()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Set time to 2 minutes ago.
+    client.t0 -= 120
+    with qtbot.waitSignal(client.timer.timeout, timeout=5000):
+        ipyconsole.set_elapsed_time(client)
+    assert ('00:02:00' in client.time_label.text() or
+            '00:02:01' in client.time_label.text())
+
+    # Wait for a second to pass, to ensure timer is counting up
+    with qtbot.waitSignal(client.timer.timeout, timeout=5000):
+        pass
+    assert ('00:02:01' in client.time_label.text() or
+            '00:02:02' in client.time_label.text())
+
+    # Make previous time later than current time.
+    client.t0 += 2000
+    with qtbot.waitSignal(client.timer.timeout, timeout=5000):
+        pass
+    assert '00:00:00' in client.time_label.text()
+
+    client.timer.timeout.disconnect(client.show_time)
+
+
 if __name__ == "__main__":
     pytest.main()
